/* --------------------------------------------------------------------------
        DNSInfoConfig.cpp

        Copyright (C) 2004 Jason Weiler, All Rights Reserved
   -------------------------------------------------------------------------- */

#include <windows.h>
#include <string.h>

#include "MacroUtils.h"

#include "DNSInfoConfig.h"

bool DNSInfoConfig::ParseCommandLine(int argc, const char * argv[])
{
	// iterate all the arguments and attempt to parse them
	int Idx=1;
	while (Idx < argc)
	{
		if ( ('/' == argv[Idx][0]) || ('-' == argv[Idx][0]) )
		{
			const char * pszSwitch = argv[Idx]+1;
			
			if (_strnicmp(pszSwitch, "svr", DEFSTR_LEN("svr")) == 0)
			{
				++Idx;
				// make sure the name-server IP is really an IP
				unsigned long ulTmpServer = inet_addr(argv[Idx]);
				if (INADDR_NONE != ulTmpServer)
				{
					m_DNSServers.AddServer(ulTmpServer);
					m_bSpecifyDNSServer = true;
				}
				else
				{
					printf("Specified server \"%s\" is invalid\n", argv[Idx]);
				}
			}
			else if (_strnicmp(pszSwitch, "type", DEFSTR_LEN("type")) == 0)
			{
				++Idx;
				// the user is specifying an explicit request type.
				m_bExcplicitType = true;
				// search the g_QueryTypeStr for the requested question

				m_QueryType = GetQueryTypeVal(argv[Idx]);
			}
			else if (_strnicmp(pszSwitch, "recurse", DEFSTR_LEN("recurse")) == 0)
			{
				// change the recursion setting to ON
				m_bUseRecursion = true;
			}
			else if (_strnicmp(pszSwitch, "norecurse", DEFSTR_LEN("norecurse")) == 0)
			{
				// change the recursion setting to OFF
				m_bUseRecursion = false;
			}
			else if ((_strnicmp(pszSwitch, "?", DEFSTR_LEN("?")) == 0) ||
					 (_strnicmp(pszSwitch, "help", DEFSTR_LEN("help")) == 0) )
			{
				// the requested help, so give it to them and exit.
				return false;
			}
			else
			{
				printf("Unknown argument: %s\n", argv[Idx][0]);
				return false;
			}
		}
		else 
		{
			//  not a switch - assume it is the host target
			if (true == m_wsQueryAddr.empty())
			{
				m_wsQueryAddr = argv[Idx];
			}
		}

		++Idx;
	}

	// --------------------------------------------
	// Begin verifying the arguments we just parsed

	// Did we get a target?
	if (true == m_wsQueryAddr.empty())
	{
		// we can't continue without an address to query
		fputs("Missing an address to query\n", stderr);
		return false;
	}

	// if the user asked for a PTR record, then the request
	//   needs to be an IP address.
	if (QUERY_TYPE_PTR == m_QueryType)
	{
		unsigned long ulTmpAddr = inet_addr(m_wsQueryAddr.c_str());
		
		if (INADDR_NONE == ulTmpAddr)
		{
			fputs("PTR Queries require an IP address\n", stderr);
			return false;
		}

		// it looks good, so we need to put it in the proper form
		char szTmpAddr[32];
		wsprintf(szTmpAddr, "%lu.%lu.%lu.%lu.in-addr.arpa",(ulTmpAddr & 0xFF000000)>>24,
			 (ulTmpAddr & 0x00FF0000)>>16, (ulTmpAddr & 0x0000FF00)>>8, ulTmpAddr & 0x000000FF);

		m_wsQueryAddr = szTmpAddr;

	}

	// if the user didn't specify a type and the target is an IP, then use a PTR request.
	if (m_bExcplicitType == false)
	{
		// if we didn't get an EXPLICIT type and our input address is an IP address,
		//   we should revert to using a PTR lookup instead of an A query
		unsigned long ulTmpAddr = inet_addr(m_wsQueryAddr.c_str());
		
		if (INADDR_NONE != ulTmpAddr)
		{
			// it looks good, so we need to put it in the proper form
			char szTmpAddr[32];
			wsprintf(szTmpAddr, "%lu.%lu.%lu.%lu.in-addr.arpa",(ulTmpAddr & 0xFF000000)>>24,
				 (ulTmpAddr & 0x00FF0000)>>16, (ulTmpAddr & 0x0000FF00)>>8, ulTmpAddr & 0x000000FF);
			
			m_wsQueryAddr = szTmpAddr;
		}
	}


	return true;
}


