/* --------------------------------------------------------------------------
        DNSInfoConfig.cpp

        Copyright (C) 2004 Jason Weiler, All Rights Reserved
   -------------------------------------------------------------------------- */

#ifndef _DNSINFOCONFIG_H_
#define _DNSINFOCONFIG_H_

#include "DNSServerList.h"
#include "DNSInfoConstants.h"

class DNSInfoConfig
{
public:
	DNSInfoConfig() :
		m_bSpecifyDNSServer(false),
		m_bUseRecursion(true),					// default to using server-recursion
		m_bExcplicitType(false),				// Did the user explicitly specify a type?
		m_QueryType(QUERY_TYPE_A),			// default to a standard name resolution
		m_ucQueryOpcode(QUERY_OPCODE_STD),	// default to the standard query
		m_ucQueryClass(QUERY_CLASS_IN)		// default to the standard query
		{}
		
	virtual ~ DNSInfoConfig() {}

	bool GetSpecifyDNSServer() const { return m_bSpecifyDNSServer; }
	bool GetUseRecursion() const { return m_bUseRecursion; }
	bool GetExplicitType() const { return m_bExcplicitType; }


	EQueryType GetQueryType() const { return m_QueryType; }
	EQueryOpcode GetQueryOpcode() const { return m_ucQueryOpcode; }
	EQueryClass GetQueryClass() const { return m_ucQueryClass; }

	const std::string & GetQueryAddress() const { return m_wsQueryAddr; }
	
	const DNSServerList & GetDNSServerList() const { return m_DNSServers; }

	bool ParseCommandLine(int argc, const char * argv[]);
	
private:

	bool m_bSpecifyDNSServer;
	bool m_bUseRecursion;	
	bool m_bExcplicitType;

	EQueryType m_QueryType;
	EQueryOpcode m_ucQueryOpcode;
	EQueryClass m_ucQueryClass;

	std::string m_wsQueryAddr;
	DNSServerList m_DNSServers;
};

#endif // _DNSINFOCONFIG_H_

