/* --------------------------------------------------------------------------
        DNSInfoConstants.h

        Copyright (C) 2004 Jason Weiler, All Rights Reserved
   -------------------------------------------------------------------------- */

#ifndef _DNSINFOCONSTANTS_H_
#define _DNSINFOCONSTANTS_H_

#include "MacroUtils.h"

const unsigned short DNS_DEFAULT_PORT   = (53);

// -------------------------
// DNS Question/Answer types
enum EQueryType
{
	  QUERY_TYPE_A			= (1)  // a host address
	, QUERY_TYPE_NS		= (2)  // an authoritative name server
	, QUERY_TYPE_MD		= (3)  // a mail destination (Obsolete - use MX)
	, QUERY_TYPE_MF		= (4)  // a mail forwarder (Obsolete - use MX)
	, QUERY_TYPE_CNAME	= (5)  // the canonical name for an alias
	, QUERY_TYPE_SOA		= (6)  // marks the start of a zone of authority
	, QUERY_TYPE_MB		= (7)  // a mailbox domain name (EXPERIMENTAL)
	, QUERY_TYPE_MG		= (8)  // a mail group member (EXPERIMENTAL)
	, QUERY_TYPE_MR		= (9)  // a mail rename domain name (EXPERIMENTAL)
	, QUERY_TYPE_NULL		= (10) // a null RR (EXPERIMENTAL)
	, QUERY_TYPE_WKS		= (11) // a well known service description
	, QUERY_TYPE_PTR		= (12) // a domain name pointer
	, QUERY_TYPE_HINFO		= (13) // host info
	, QUERY_TYPE_MINFO	= (14) // mailbox or mail list information
	, QUERY_TYPE_MX		= (15) // mail exchange
	, QUERY_TYPE_TXT		= (16) // text strings
	// Question only
	, QUERY_TYPE_AXFR		= (252) // request zone change
	, QUERY_TYPE_ANY		= (255) // any
};

struct StrValAssoc_t
{
	char * String;
	int Value;
};

const StrValAssoc_t g_QueryTypes[] = {
					 	  {"???",		0}					// Use this entry for unknown values
						, {"A",		QUERY_TYPE_A}
						, {"NS",		QUERY_TYPE_NS}
						, {"MD",		QUERY_TYPE_MD}
						, {"MF",		QUERY_TYPE_MF}
						, {"CNAME",	QUERY_TYPE_CNAME}
						, {"SOA",		QUERY_TYPE_SOA}
						, {"MB",		QUERY_TYPE_MB}
						, {"MG",		QUERY_TYPE_MG}
						, {"MR",		QUERY_TYPE_MR}
						, {"NULL",		QUERY_TYPE_NULL}
						, {"WKS",		QUERY_TYPE_WKS}
						, {"PTR",		QUERY_TYPE_PTR}
						, {"HINFO",	QUERY_TYPE_HINFO}
						, {"MINFO",	QUERY_TYPE_MINFO}
						, {"MX",		QUERY_TYPE_MX}
						, {"TXT",		QUERY_TYPE_TXT}
						, {"AXFR",		QUERY_TYPE_AXFR}
						, {"ANY",		QUERY_TYPE_ANY}
					};

const unsigned long g_QueryTypesLen = ARRAY_LEN(g_QueryTypes);



// -----------------------
// Network class constants
enum EQueryClass
{
	  QUERY_CLASS_IN		= (1)	// internet
	, QUERY_CLASS_CH		= (3)	// the CHAOS class
	, QUERY_CLASS_HS		= (4)	// 4 Hesiod [Dyer 87]
};

const StrValAssoc_t g_QueryClasses[] = {
						  {"???",		0}					// Use this entry for unknown values
						, {"IN",		QUERY_CLASS_IN}
						, {"CH",		QUERY_CLASS_CH}
						, {"HS",		QUERY_CLASS_HS}
					};
						

const unsigned long g_QueryClassesLen = ARRAY_LEN(g_QueryClasses);

// --------------------
// DNS Opcode constants
enum EQueryOpcode
{
	  QUERY_OPCODE_STD	= 0	// Standard Query
	, QUERY_OPCODE_INV	= 1	// Inverse Query
	, QUERY_OPCODE_SSR	= 2	// Server Status Request
};

const StrValAssoc_t g_QueryOpcodes[] = {
						  {"STD",		QUERY_OPCODE_STD}
						, {"INV",		QUERY_OPCODE_INV}
						, {"SSR",		QUERY_OPCODE_SSR}
					};
						

const unsigned long g_QueryOpcodesLen = ARRAY_LEN(g_QueryOpcodes);

// ---------------------
// DNS Parsing constants
#define DNS_NAME_COMPRESSION_FLAG  (0xC000)
#define NOT_DNS_NAME_COMPRESSION_MASK  (~DNS_NAME_COMPRESSION_FLAG)

extern const char * GetQueryTypeStr(EQueryType qType);
extern EQueryType GetQueryTypeVal(const char * str);
extern const char * GetQueryClassStr(EQueryClass qClass);
extern EQueryClass GetQueryClassVal(const char * str);
extern const char * GetQueryOpcodeStr(EQueryOpcode idx);
extern EQueryOpcode GetQueryOpcodeVal(const char * str);

#endif // _DNSINFOCONSTANTS_H_

