/* --------------------------------------------------------------------------
        DNSInfoTypes.h

        Copyright (C) 2004 Jason Weiler, All Rights Reserved
   -------------------------------------------------------------------------- */

#ifndef _DNSINFOTYPES_H_
#define _DNSINFOTYPES_H_

#include <string>


// RCode		xxxx.... ........
// zero			....xxx. ........
// RA			.......x ........
// RD			........ x.......
// TC			........ .x......
// AA			........ ..x.....
// Opcode		........ ...xxxx.
// QR			........ .......x


#define FLAGS_GetRCode(X)		(((X) >> 12) & 0xF000)

#define FLAGS_GetRA(X)				(((X) >>  9) & 0x0E00)
#define FLAGS_GetRD(X)				(((X) >>  8) & 0x0100)
#define FLAGS_GetTC(X)				(((X) >>  7) & 0x0080)
#define FLAGS_GetAA(X)				(((X) >>  6) & 0x0040)
#define FLAGS_GetOpcode(X)		(((X) >>  1) & 0x001E)
#define FLAGS_GetQR(X)				( (X) 		 & 0x0001)

#define FLAGS_SetRA(X, VAL)		((X) |= ((VAL) << 9))
#define FLAGS_SetRD(X, VAL)		((X) |= ((VAL) << 8))
#define FLAGS_SetTC(X, VAL)		((X) |= ((VAL) << 7))
#define FLAGS_SetAA(X, VAL)		((X) |= ((VAL) << 6))
#define FLAGS_SetOpcode(X, VAL)	((X) |= ((VAL) << 1))
#define FLAGS_SetQR(X, VAL)		((X) |=  (VAL)      )



struct DNSPacketHeader_t
{
	unsigned short Identification;
	unsigned short usFlags;
	unsigned short usNumQuestions;
	unsigned short usNumAnswerRRs;
	unsigned short usNumAuthorityRRs;
	unsigned short usNumAdditionalRRs;
};
const unsigned long DNSPacketHeaderSize = (12);


// This DOES NOT include the variable-length portion of the question
struct DNSQuestion_t
{
	unsigned short usType;
	unsigned short usClass;
};
const unsigned long DNSQuestionSize = (4);


// This DOES NOT include the variable-length portion of the resource
struct DNSResource_t
{
	unsigned short usType;
	unsigned short usClass;
	unsigned long ulTTL;
	unsigned short usRDataLen;
	// Warning! This structure actually pads out to 12 bytes on Win32.
	//  Do not use sizeof(DNSResource_t)
};
const unsigned long DNSResourceSize = (10);

struct DNSSOAResource_t
{
	unsigned long ulSerial;
	unsigned long ulRefresh;
	unsigned long ulRetry;
	unsigned long ulExpire;
	unsigned long ulMinimum;
};
const unsigned long DNSSOAResourceSize	(20);

struct DNSRequest_t
{
	std::string sAddress;		// The address we want to look up
	unsigned long ulServer;	// 32-bit IP address of the DNS server
	unsigned short usType;	// The DNS question type to be asked
	unsigned short usClass;	// For now, this is always 'internet'
	unsigned char ucOpcode;	// This should normally be 0 (standard)
	
	bool bUseRecursion;		// Should the DNS server do recursion?
};

#endif // _DNSINFOTYPES_H_

