/* --------------------------------------------------------------------------
        DNSPacketParsing.cpp

        Copyright (C) 2010 Jason Weiler, All Rights Reserved
   -------------------------------------------------------------------------- */

#include "DNSInfoTypes.h"
#include "DNSPacketParsing.h"
#include "Winsock.h"

unsigned long ParseDNSPacketHeader(const unsigned char * parsePtr, DNSPacketHeader_t & dnsHdr)
{
	if (parsePtr == NULL)
	{
		return 0;
	}

	memcpy(&dnsHdr, parsePtr, DNSPacketHeaderSize);

	// all the fields are in network-byte order, put it in local-byte order
	dnsHdr.usFlags = ntohs(dnsHdr.usFlags);
	dnsHdr.usNumQuestions = ntohs(dnsHdr.usNumQuestions);
	dnsHdr.usNumAnswerRRs = ntohs(dnsHdr.usNumAnswerRRs);
	dnsHdr.usNumAuthorityRRs = ntohs(dnsHdr.usNumAuthorityRRs);
	dnsHdr.usNumAdditionalRRs = ntohs(dnsHdr.usNumAdditionalRRs);

	return DNSPacketHeaderSize;
}

unsigned long ParseDNSQuestion(const unsigned char * parsePtr, DNSQuestion_t & dnsQstn)
{
	if (parsePtr == NULL)
	{
		return 0;
	}

	memcpy(&dnsQstn, parsePtr, DNSQuestionSize);


	// all the fields are in network-byte order, put it in local-byte order
	dnsQstn.usType = ntohs(dnsQstn.usType);
	dnsQstn.usClass = ntohs(dnsQstn.usClass);

	return DNSQuestionSize;
}

unsigned long ParseDNSResource(const unsigned char * parsePtr, DNSResource_t & dnsRsrc)
{
	if (parsePtr == NULL)
	{
		return 0;
	}

	memcpy(&dnsRsrc, parsePtr, DNSResourceSize);

	// all the fields are in network-byte order, put it in local-byte order

	dnsRsrc.usType = ntohs(dnsRsrc.usType);
	dnsRsrc.usClass = ntohs(dnsRsrc.usClass);
	dnsRsrc.ulTTL = ntohl(dnsRsrc.ulTTL);
	dnsRsrc.usRDataLen = ntohs(dnsRsrc.usRDataLen);

	return DNSResourceSize;
}

unsigned long ParseSOAResourceData(const unsigned char * parsePtr, DNSSOAResource_t & soaRec)
{
	if (parsePtr == NULL)
	{
		return 0;
	}

	memcpy(&soaRec, parsePtr, DNSSOAResourceSize);

	// all the fields are in network-byte order, put them in local-byte order

	soaRec.ulSerial = ntohl(soaRec.ulSerial);
	soaRec.ulRefresh = ntohl(soaRec.ulRefresh);
	soaRec.ulRetry = ntohl(soaRec.ulRetry);
	soaRec.ulExpire = ntohl(soaRec.ulExpire);
	soaRec.ulMinimum = ntohl(soaRec.ulMinimum);
	
	return DNSSOAResourceSize;
}

