/* --------------------------------------------------------------------------
        DataBuffer.cpp

        Copyright (C) 2004 Jason Weiler, All Rights Reserved
   -------------------------------------------------------------------------- */

#include <windows.h>

#include "DataBuffer.h"

const unsigned long DEFAULT_GROW_SIZE  = (256);

CDataBuffer::CDataBuffer() :
m_Data(NULL),
m_DataLen(0),
m_BufferSize(0),
m_GrowSize(DEFAULT_GROW_SIZE)
{
	return;
}

CDataBuffer::~CDataBuffer()
{
	delete [] m_Data;
	m_Data = NULL;
	return;
}

unsigned char * CDataBuffer::GetDataPtr(size_t * outLen)
{
	if (m_DataLen == 0)
	{
		return NULL;
	}

	if (outLen != NULL)
	{
		*outLen = m_DataLen;
	}

	return m_Data;
}

const unsigned char * CDataBuffer::GetDataPtr(size_t * outLen) const
{
	if (m_DataLen == 0)
	{
		return NULL;
	}

	if (outLen != NULL)
	{
		*outLen = m_DataLen;
	}

	return m_Data;
}

bool CDataBuffer::Append(const unsigned char * newData, size_t newDataLen)
{
	if (newData == NULL)
	{
		// NULL data...trivial success
		return true;
	}

	if (newDataLen == 0)
	{
		// empty data...trivial success
		return true;
	}

	// so we have valid data to add. Do we have enough space?
	if (m_BufferSize <= m_DataLen + newDataLen)
	{
		// we need to realloc
		unsigned char * tmpBuf = new unsigned char[m_DataLen + newDataLen + m_GrowSize];
		if (tmpBuf == NULL)
		{
			return false;
		}
		
		// Copy the old
		if (m_DataLen > 0)
		{
			memcpy(tmpBuf, m_Data, m_DataLen);
		}

		delete [] m_Data;
		m_Data = tmpBuf;
		
		tmpBuf = NULL;
		
		m_BufferSize = m_DataLen + newDataLen+m_GrowSize;
		m_GrowSize = m_BufferSize<<1;
	}

	// append the new
	memcpy(m_Data + m_DataLen, newData, newDataLen);
	m_DataLen += newDataLen;

	return true;
}

bool CDataBuffer::Set(const unsigned char * newData, size_t newDataLen)
{
	// clear out the old data
	m_DataLen = 0;
	memset(m_Data, 0, m_BufferSize);

	// set the new data
	return Append(newData, newDataLen);
}

void CDataBuffer::Clear()
{
	delete [] m_Data;
	m_Data = NULL;
	
	m_DataLen = 0;
	m_BufferSize = 0;
	m_GrowSize = DEFAULT_GROW_SIZE;

	return;
}

bool CDataBuffer::CheckBufferSize(unsigned long reqBytes)
{
	if (m_BufferSize > m_DataLen + reqBytes)
	{
		// trival success
		return true;
	}
	// we need to realloc
	unsigned char * tmpBuf = new unsigned char[m_DataLen + reqBytes + m_GrowSize + 1];
	if (tmpBuf == NULL)
	{
		// alloc failure
		return false;
	}
	
	// Copy the old
	if (m_DataLen > 0)
	{
		memcpy(tmpBuf, m_Data, m_DataLen);
	}

	delete [] m_Data;
	m_Data = tmpBuf;
	
	tmpBuf = NULL;
	
	m_BufferSize = m_DataLen + reqBytes + m_GrowSize;
	m_GrowSize = (m_BufferSize)>>1;

	// realloced success
	return true;
}

