/* --------------------------------------------------------------------------
        TCPTransactionNetwork.cpp

        Copyright (C) 2004 Jason Weiler, All Rights Reserved
   -------------------------------------------------------------------------- */

#include <windows.h>
#include "IPTransactionNetwork.h"
#include "DataBuffer.h"

#include "TCPTransactionNetwork.h"

// 30-second timeout on RECVs
unsigned long TCPTransactionNetwork::RECV_TIMEOUT_LENGTH = 30000;

bool TCPTransactionNetwork::Connect()
{
	if ((INADDR_NONE == m_ulServerIP) || (0 == m_usPort))
	{
		return false;
	}


	// Create a TCP/IP stream socket
	m_Socket = socket(AF_INET,			// Address family
					  SOCK_STREAM,		// Socket type
					  IPPROTO_TCP);		// Protocol

	if (INVALID_SOCKET == m_Socket)
	{
		return false;
	}

	// Fill in the socket address info for the host/port that we want to connect to
	m_SocketAddr.sin_family = AF_INET;
	m_SocketAddr.sin_port = htons(m_usPort);

	m_SocketAddr.sin_addr.s_addr = m_ulServerIP;

	if (SOCKET_ERROR == connect(m_Socket, (const SOCKADDR*)&m_SocketAddr, sizeof(m_SocketAddr)))
	{
		return false;
	}

	return true;
}

void TCPTransactionNetwork::Disconnect()
{
	if (INVALID_SOCKET != m_Socket)
	{
		shutdown(m_Socket, 0x02);
		closesocket(m_Socket);
		m_Socket = INVALID_SOCKET;
	}

	return;
}

bool TCPTransactionNetwork::SendTransaction(const CDataBuffer & sendBuf, CDataBuffer & replyBuf)
{
	size_t sendDataLen;
	const unsigned char * sendData = sendBuf.GetDataPtr(&sendDataLen);

	long rc = send(m_Socket, (const char*)sendData, sendDataLen,0);
	if (SOCKET_ERROR == rc)
	{
		return false;
	}

	unsigned long pendingDataLen = 0;
	unsigned long recvTimeout = GetTickCount() + RECV_TIMEOUT_LENGTH;

	do
	{
		rc = ioctlsocket(m_Socket, FIONREAD, &pendingDataLen);
		Sleep(1);
	}
	while((0 == pendingDataLen) && (0 == rc) && (GetTickCount() < recvTimeout));

	if (0 == pendingDataLen)
	{
		return false;
	}


	unsigned char * tmpReply = NULL;
	tmpReply = new unsigned char[pendingDataLen];
	if (NULL == tmpReply)
	{
		return false;
	}

	rc = recv(m_Socket, (char*)tmpReply, pendingDataLen, 0);
	if (SOCKET_ERROR == rc)
	{
		return false;
	}

	replyBuf.Set(tmpReply, rc);

	delete [] tmpReply;
	tmpReply = NULL;

	return true;
}


