/* --------------------------------------------------------------------------
        UDPTransactionNetwork.cpp

        Copyright (C) 2004 Jason Weiler, All Rights Reserved
   -------------------------------------------------------------------------- */

#include <windows.h>
#include "IPTransactionNetwork.h"
#include "DataBuffer.h"

#include "UDPTransactionNetwork.h"

// 30-second timeout on RECVs
unsigned long UDPTransactionNetwork::RECVFROM_TIMEOUT_LENGTH = 30000;

bool UDPTransactionNetwork::Connect()
{
	if ((INADDR_NONE == m_ulServerIP) || (0 == m_usPort))
	{
		return false;
	}


	// Create a UDP/IP datagram socket
	m_Socket = socket(AF_INET,			// Address family
					  SOCK_DGRAM,		// Socket type
					  IPPROTO_UDP);		// Protocol

	if (INVALID_SOCKET == m_Socket)
	{
		return false;
	}

	// Fill in the socket address info for the host/port that we want to connect to
	m_SocketAddr.sin_family = AF_INET;
	m_SocketAddr.sin_port = htons(m_usPort);

	m_SocketAddr.sin_addr.s_addr = m_ulServerIP;

	return true;
}

void UDPTransactionNetwork::Disconnect()
{
	if (INVALID_SOCKET != m_Socket)
	{
		shutdown(m_Socket, 0x02);
		closesocket(m_Socket);
		m_Socket = INVALID_SOCKET;
	}

	return;
}

bool UDPTransactionNetwork::SendTransaction(const CDataBuffer & sendBuf,
												CDataBuffer & replyBuf)
{
	size_t sendDataLen;
	const unsigned char * sendData = sendBuf.GetDataPtr(&sendDataLen);

	long rc = sendto(m_Socket, (const char*)sendData, sendDataLen,
					 0, (LPSOCKADDR)&m_SocketAddr, sizeof(m_SocketAddr));
	if (SOCKET_ERROR == rc)
	{
		return false;
	}

	int outSize = sizeof(SOCKADDR_IN);

	unsigned long pendingDataLen = 0;
	unsigned long recvTimeout = GetTickCount() + RECVFROM_TIMEOUT_LENGTH;

	do
	{
		rc = ioctlsocket(m_Socket, FIONREAD, &pendingDataLen);
		Sleep(1);
	}
	while((0 == pendingDataLen) && (0 == rc) && (GetTickCount() < recvTimeout));

	if (pendingDataLen == 0)
	{
		return false;
	}

	unsigned char * tmpReply = NULL;
	tmpReply = new unsigned char[pendingDataLen];
	if (NULL == tmpReply)
	{
		return false;
	}

	rc = recvfrom(m_Socket, (char*)tmpReply, pendingDataLen,
					0, (LPSOCKADDR)&m_SocketAddr, &outSize);
	if (SOCKET_ERROR == rc)
	{
		return false;
	}

	replyBuf.Set(tmpReply, rc);

	delete [] tmpReply;
	tmpReply = NULL;

	return true;
}



